==> commerce_funds-2.1.0.tar.gz: commerce_funds/tests/src/Functional/DefaultFormsTest.php <==
<?php

namespace Drupal\Tests\commerce_funds\Functional;

use Drupal\Tests\commerce\Functional\CommerceBrowserTestBase;
use Drupal\Tests\commerce_funds\Traits\FundsTrait;
use Drupal\user\Entity\User;

/**
 * Tests transaction default forms.
 *
 * @group commerce_funds
 */
class DefaultFormsTest extends CommerceBrowserTestBase {

  use FundsTrait;

  /**
   * Set default theme.
   *
   * @var string
   */
  protected $defaultTheme = 'stark';

  /**
   * Issue with views.view.commerce_funds_transactions schema.
   *
   * @var bool
   *
   * @todo Remove when issue fixed: https://www.drupal.org/project/commerce_funds/issues/3219941
   */
  protected $strictConfigSchema = FALSE;

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = [
    'commerce_funds',
    'commerce_exchanger',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp() : void {
    parent::setUp();

    $this->firstUser = $this->drupalCreateUser([
      'deposit funds',
      'access checkout',
      'transfer funds',
      'create escrow payment',
      'withdraw funds',
      'convert currencies',
    ]);
    $this->drupalLogin($this->firstUser);
    $this->secondUser = $this->drupalCreateUser();

    $this->paymentGateway = $this->createEntity('commerce_payment_gateway', [
      'id' => 'manual',
      'label' => 'Manual',
      'plugin' => 'manual',
      'collect_billing_information' => FALSE,
      'status' => 1,
    ]);
    $this->transactionManager = \Drupal::service('commerce_funds.transaction_manager');
    $this->productManager = \Drupal::service('commerce_funds.product_manager');
    $this->configFactory = \Drupal::configFactory();
    $this->webAssert = $this->assertSession();
  }

  /**
   * Tests Funds deposit form.
   */
  public function testFundsDepositFormSubmission() {
    $this->drupalGet('user/funds/deposit');
    $this->webAssert->statusCodeEquals(200);

    $this->submitForm([
      'amount' => 12.5,
      'currency' => 'USD',
    ], 'Next');

    // Test checkout redirection.
    $this->assertEquals($this->getUrl(), $this->getAbsoluteUrl('/checkout/1/order_information'));
    // Product created + added to order.
    $this->webAssert->pageTextContains('Deposit 12.5 USD');

    // Order information.
    $this->submitForm([
      'payment_information[billing_information][address][0][address][administrative_area]' => 'AL',
      'payment_information[billing_information][address][0][address][given_name]' => 'firstUser',
      'payment_information[billing_information][address][0][address][family_name]' => 'firstUser',
      'payment_information[billing_information][address][0][address][address_line1]' => 'Somewhere',
      'payment_information[billing_information][address][0][address][locality]' => 'Somewhere',
      'payment_information[billing_information][address][0][address][postal_code]' => '35242',
    ], 'Continue to review');

    // Review.
    $this->submitForm([], 'Pay and complete purchase');

    // Funds added to balance.
    $this->webAssert->pageTextContains('12.50 USD have been added to your account balance.');
    // Assert deposit is in balance.
    $this->assertEquals($this->transactionManager->loadAccountBalance($this->firstUser)['USD'], 12.5);
  }

  /**
   * Tests Funds transfer and escrow forms.
   */
  public function testFundsTransferEscrowFormSubmission() {
    $forms = [
      'transfer' => 'Transfer funds',
      'escrow' => 'Create escrow',
    ];

    foreach ($forms as $transaction_type => $button) {
      $confirmation_message = $transaction_type == 'transfer' ? 'You have transfered $12.5 USD to' : 'Escrow payment of $12.5 USD successfully created to';
      $this->drupalGet('user/funds/' . $transaction_type);
      $this->webAssert->statusCodeEquals(200);

      // Form values.
      $values = [
        'amount' => 12.5,
        'currency' => 'USD',
        'username' => 'secondUser (' . $this->secondUser->id() . ')',
      ];

      // Test with no funds.
      $this->submitForm($values, $button);
      $this->webAssert->pageTextContains('Not enough funds to cover this ' . $transaction_type);

      // Deposit funds.
      $this->depositFunds(25.25);

      // Test transaction is well executed.
      $this->submitForm($values, $button);
      $this->webAssert->pageTextContains($confirmation_message . ' ' . $this->secondUser->getAccountName());
      // Assert balance is 25.25 - 12.5.
      $this->assertEquals($this->transactionManager->loadAccountBalance($this->firstUser)['USD'], 12.75);

      // Set fees.
      $this->configFactory->getEditable('commerce_funds.settings')->set('fees', [$transaction_type . '_rate' => '2'])->save();
      $this->submitForm($values, $button);
      $this->webAssert->pageTextContains($confirmation_message . ' ' . $this->secondUser->getAccountName() . ' (fees: $0.25 USD)');
      // Assert user balance is 0.
      $this->assertEquals($this->transactionManager->loadAccountBalance($this->firstUser)['USD'], 0);
      // Assert site balance was updated from $0.25.
      $this->assertEquals($this->transactionManager->loadAccountBalance(User::load(1))['USD'], 0.25);
      // Assert second user balance.
      if ($transaction_type == 'transfer') {
        $this->assertEquals($this->transactionManager->loadAccountBalance($this->secondUser)['USD'], 25);
      }
    }
  }

  /**
   * Tests Funds converter form.
   */
  public function testFundsConverterFormSubmission() {
    // Form values.
    $values = [
      'currency_left' => 'USD',
      'amount' => 10,
      'currency_right' => 'EUR',
    ];
    // Add another currency.
    $this->createEntity('commerce_currency', [
      'name' => 'Euro',
      'currencyCode' => 'EUR',
      'symbol' => '€',
      'numericCode' => '978',
      'fractionDigits' => 2,
    ]);
    // Add another currency.
    $this->createEntity('commerce_currency', [
      'name' => 'Kuna',
      'currencyCode' => 'HRK',
      'symbol' => 'HRK',
      'numericCode' => '191',
      'fractionDigits' => 2,
    ]);

    // Test with no rates.
    $this->drupalGet('user/funds/converter');
    $this->webAssert->statusCodeEquals(200);
    $this->webAssert->pageTextContains('Sorry, no exchange rates are available at the moment.');

    // Add exchange rates.
    $exchange_rates = $this->createEntity('commerce_exchange_rates', [
      'id' => 'manual',
      'plugin' => 'manual',
    ]);
    $config_name = $exchange_rates->getExchangerConfigName();
    // Set exchange rate rates.
    $this->configFactory->getEditable($config_name)->set('rates', [
      'EUR' => ['USD' => ['value' => 1.20]],
      'USD' => ['EUR' => ['value' => 0.8]],
    ])->save();
    // Set exchange rate plugin to be used.
    $this->configFactory->getEditable('commerce_funds.settings')->set('exchange_rate_provider', 'manual')->save();

    $this->drupalGet('user/funds/converter');

    // Try non set exchange rates.
    $this->submitForm([
      'currency_left' => 'USD',
      'amount' => 10,
      'currency_right' => 'HRK',
    ], 'Convert funds');
    $this->webAssert->pageTextContains('Operation impossible. No exchange rates found.');

    // Deposit funds.
    $this->depositFunds(100);

    // Test that transaction is well executed.
    $this->submitForm($values, 'Convert funds');
    $this->webAssert->pageTextContains('$10 USD converted into €8 EUR.');
    $user_balance = $this->transactionManager->loadAccountBalance($this->firstUser);
    // Assert balance udpates.
    $this->assertEquals($user_balance['USD'], 90);
    $this->assertEquals($user_balance['EUR'], 8);
  }

  /**
   * Tests Funds withdrawal request form.
   */
  public function testFundsWithdrawFormSubmission() {
    // Set a withdrawal method.
    $this->configFactory->getEditable('commerce_funds.settings')->set('withdrawal_methods', ['paypal' => 'paypal'])->save();

    $this->drupalGet('user/funds/withdraw');
    $this->webAssert->statusCodeEquals(200);

    // Form values.
    $values = [
      'amount' => 12.5,
      'currency' => 'USD',
      'methods' => 'paypal',
    ];

    // Test with no funds.
    $this->submitForm($values, 'Submit request');
    $this->webAssert->pageTextContains('Your available balance is 0 USD.');
    $this->webAssert->pageTextContains('Please enter your details for this withdrawal method first.');

    // Deposit funds.
    $this->depositFunds(100);
    \Drupal::service('user.data')->set('commerce_funds', $this->firstUser->id(), 'paypal', 'firstUser@nomail.com');

    // Send withdrawal request.
    $this->submitForm($values, 'Submit request');
    $this->webAssert->pageTextContains('Withdrawal request sent.');
    // Assert balance didn't change.
    $this->assertEquals($this->transactionManager->loadAccountBalance($this->firstUser)['USD'], 100);

    // Set fees.
    $this->configFactory->getEditable('commerce_funds.settings')->set('fees', ['withdraw_paypal_rate' => '2'])->save();
    $this->submitForm($values, 'Submit request');
    $this->webAssert->pageTextContains('Withdrawal request sent. An extra commission of $0.25 USD will be apllied to your withraw.');
    // Assert balance didn't change.
    $this->assertEquals($this->transactionManager->loadAccountBalance($this->firstUser)['USD'], 100);
  }

}

==> commerce_funds-2.1.3.tar.gz: commerce_funds/tests/src/Functional/DefaultFormsTest.php <==
<?php

namespace Drupal\Tests\commerce_funds\Functional;

use Drupal\Tests\commerce\Functional\CommerceBrowserTestBase;
use Drupal\Tests\commerce_funds\Traits\FundsTrait;
use Drupal\user\Entity\User;

/**
 * Tests transaction default forms.
 *
 * @group commerce_funds
 */
class DefaultFormsTest extends CommerceBrowserTestBase {

  use FundsTrait;

  /**
   * Set default theme.
   *
   * @var string
   */
  protected $defaultTheme = 'stark';

  /**
   * Issue with views.view.commerce_funds_transactions schema.
   *
   * @var bool
   *
   * @todo Remove when issue fixed: https://www.drupal.org/project/commerce_funds/issues/3219941
   */
  protected $strictConfigSchema = FALSE;

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = [
    'commerce_funds',
    'commerce_exchanger',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp() : void {
    parent::setUp();

    $this->firstUser = $this->drupalCreateUser([
      'deposit funds',
      'access checkout',
      'transfer funds',
      'create escrow payment',
      'withdraw funds',
      'convert currencies',
    ]);
    $this->drupalLogin($this->firstUser);
    $this->secondUser = $this->drupalCreateUser();

    $this->paymentGateway = $this->createEntity('commerce_payment_gateway', [
      'id' => 'manual',
      'label' => 'Manual',
      'plugin' => 'manual',
      'collect_billing_information' => FALSE,
      'status' => 1,
    ]);
    $this->transactionManager = \Drupal::service('commerce_funds.transaction_manager');
    $this->productManager = \Drupal::service('commerce_funds.product_manager');
    $this->configFactory = \Drupal::configFactory();
    $this->webAssert = $this->assertSession();
  }

  /**
   * Tests Funds deposit form.
   */
  public function testFundsDepositFormSubmission() {
    $this->drupalGet('user/funds/deposit');
    $this->webAssert->statusCodeEquals(200);

    $this->submitForm([
      'amount' => 12.5,
      'currency' => 'USD',
    ], 'Next');

    // Test checkout redirection.
    $this->assertEquals($this->getUrl(), $this->getAbsoluteUrl('/checkout/1/order_information'));
    // Product created + added to order.
    $this->webAssert->pageTextContains('Deposit 12.5 USD');

    // Order information.
    $this->submitForm([
      'payment_information[billing_information][address][0][address][administrative_area]' => 'AL',
      'payment_information[billing_information][address][0][address][given_name]' => 'firstUser',
      'payment_information[billing_information][address][0][address][family_name]' => 'firstUser',
      'payment_information[billing_information][address][0][address][address_line1]' => 'Somewhere',
      'payment_information[billing_information][address][0][address][locality]' => 'Somewhere',
      'payment_information[billing_information][address][0][address][postal_code]' => '35242',
    ], 'Continue to review');

    // Review.
    $this->submitForm([], 'Pay and complete purchase');

    // Funds added to balance.
    $this->webAssert->pageTextContains('12.50 USD have been added to your account balance.');
    // Assert deposit is in balance.
    $this->assertEquals($this->transactionManager->loadAccountBalance($this->firstUser)['USD'], 12.5);
  }

  /**
   * Tests Funds transfer and escrow forms.
   */
  public function testFundsTransferEscrowFormSubmission() {
    $forms = [
      'transfer' => 'Transfer funds',
      'escrow' => 'Create escrow',
    ];

    foreach ($forms as $transaction_type => $button) {
      $confirmation_message = $transaction_type == 'transfer' ? 'You have transfered $12.5 USD to' : 'Escrow payment of $12.5 USD successfully created to';
      $this->drupalGet('user/funds/' . $transaction_type);
      $this->webAssert->statusCodeEquals(200);

      // Form values.
      $values = [
        'amount' => 12.5,
        'currency' => 'USD',
        'username' => 'secondUser (' . $this->secondUser->id() . ')',
      ];

      // Test with no funds.
      $this->submitForm($values, $button);
      $this->webAssert->pageTextContains('Not enough funds to cover this ' . $transaction_type);

      // Deposit funds.
      $this->depositFunds(25.25);

      // Test transaction is well executed.
      $this->submitForm($values, $button);
      $this->webAssert->pageTextContains($confirmation_message . ' ' . $this->secondUser->getAccountName());
      // Assert balance is 25.25 - 12.5.
      $this->assertEquals($this->transactionManager->loadAccountBalance($this->firstUser)['USD'], 12.75);

      // Set fees.
      $this->configFactory->getEditable('commerce_funds.settings')->set('fees', [$transaction_type . '_rate' => '2'])->save();
      $this->submitForm($values, $button);
      $this->webAssert->pageTextContains($confirmation_message . ' ' . $this->secondUser->getAccountName() . ' (fees: $0.25 USD)');
      // Assert user balance is 0.
      $this->assertEquals($this->transactionManager->loadAccountBalance($this->firstUser)['USD'], 0);
      // Assert site balance was updated from $0.25.
      $this->assertEquals($this->transactionManager->loadAccountBalance(User::load(1))['USD'], 0.25);
      // Assert second user balance.
      if ($transaction_type == 'transfer') {
        $this->assertEquals($this->transactionManager->loadAccountBalance($this->secondUser)['USD'], 25);
      }
    }
  }

  /**
   * Tests Funds converter form.
   */
  public function testFundsConverterFormSubmission() {
    // Form values.
    $values = [
      'currency_left' => 'USD',
      'amount' => 10,
      'currency_right' => 'EUR',
    ];
    // Add another currency.
    $this->createEntity('commerce_currency', [
      'name' => 'Euro',
      'currencyCode' => 'EUR',
      'symbol' => '€',
      'numericCode' => '978',
      'fractionDigits' => 2,
    ]);
    // Add another currency.
    $this->createEntity('commerce_currency', [
      'name' => 'Kuna',
      'currencyCode' => 'HRK',
      'symbol' => 'HRK',
      'numericCode' => '191',
      'fractionDigits' => 2,
    ]);

    // Test with no rates.
    $this->drupalGet('user/funds/converter');
    $this->webAssert->statusCodeEquals(200);
    $this->webAssert->pageTextContains('Sorry, no exchange rates are available at the moment.');

    // Add exchange rates.
    $exchange_rates = $this->createEntity('commerce_exchange_rates', [
      'id' => 'manual',
      'plugin' => 'manual',
    ]);
    $config_name = $exchange_rates->getExchangerConfigName();
    // Set exchange rate rates.
    $this->configFactory->getEditable($config_name)->set('rates', [
      'EUR' => ['USD' => ['value' => 1.20]],
      'USD' => ['EUR' => ['value' => 0.8]],
    ])->save();
    // Set exchange rate plugin to be used.
    $this->configFactory->getEditable('commerce_funds.settings')->set('exchange_rate_provider', 'manual')->save();

    $this->drupalGet('user/funds/converter');

    // Try non set exchange rates.
    $this->submitForm([
      'currency_left' => 'USD',
      'amount' => 10,
      'currency_right' => 'HRK',
    ], 'Convert funds');
    $this->webAssert->pageTextContains('Operation impossible. No exchange rates found.');

    // Deposit funds.
    $this->depositFunds(100);

    // Test that transaction is well executed.
    $this->submitForm($values, 'Convert funds');
    $this->webAssert->pageTextContains('$10 USD converted into €8 EUR.');
    $user_balance = $this->transactionManager->loadAccountBalance($this->firstUser);
    // Assert balance udpates.
    $this->assertEquals($user_balance['USD'], 90);
    $this->assertEquals($user_balance['EUR'], 8);
  }

  /**
   * Tests Funds withdrawal request form.
   */
  public function testFundsWithdrawFormSubmission() {
    // Set a withdrawal method.
    $this->configFactory->getEditable('commerce_funds.settings')->set('withdrawal_methods', ['paypal' => 'paypal'])->save();

    $this->drupalGet('user/funds/withdraw');
    $this->webAssert->statusCodeEquals(200);

    // Form values.
    $values = [
      'amount' => 12.5,
      'currency' => 'USD',
      'methods' => 'paypal',
    ];

    // Test with no funds.
    $this->submitForm($values, 'Submit request');
    $this->webAssert->pageTextContains('Your available balance is 0 USD.');
    $this->webAssert->pageTextContains('Please enter your details for this withdrawal method first.');

    // Deposit funds.
    $this->depositFunds(100);
    \Drupal::service('user.data')->set('commerce_funds', $this->firstUser->id(), 'paypal', ['paypal_email' => 'firstUser@nomail.com']);

    // Send withdrawal request.
    $this->submitForm($values, 'Submit request');
    $this->webAssert->pageTextContains('Withdrawal request sent.');
    // Assert balance didn't change.
    $this->assertEquals($this->transactionManager->loadAccountBalance($this->firstUser)['USD'], 100);

    // Set fees.
    $this->configFactory->getEditable('commerce_funds.settings')->set('fees', ['withdraw_paypal_rate' => '2'])->save();
    $this->submitForm($values, 'Submit request');
    $this->webAssert->pageTextContains('Withdrawal request sent. An extra commission of $0.25 USD will be apllied to your withraw.');
    // Assert balance didn't change.
    $this->assertEquals($this->transactionManager->loadAccountBalance($this->firstUser)['USD'], 100);
  }

}
